unit Gutenberg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, utenbergMemo, RxRichEd, ComCtrls, GutenbergTreeView,
  Menus, InsertBookMark, OleCtrls, AgentObjects_TLB, StringFns, inifiles, comobj;

type
  TMainForm = class(TForm)
    LeftPanel: TPanel;
    RightPanel: TPanel;
    GutenbergMemo: TGutenbergMemo;
    Splitter: TSplitter;
    TreeView: TGutenbergTreeView;
    PopupMenu: TPopupMenu;
    ChangeFontMenuItem: TMenuItem;
    FontDialog: TFontDialog;
    AddBookMarkMenuItem: TMenuItem;
    ReadFromCaretMenuItem: TMenuItem;
    MainMenu: TMainMenu;
    FileMenuItem: TMenuItem;
    OpenBookMenuItem: TMenuItem;
    OpenDialog: TOpenDialog;
    AgentMenuItem: TMenuItem;
    SelectAgentMenuItem: TMenuItem;
    OpenDialogAgent: TOpenDialog;
    ExitMenuClick: TMenuItem;
    SaveBooksettingsMenuItem: TMenuItem;
    ChangeSpeedMenuItem: TMenuItem;

    TreePopupMenu: TPopupMenu;
    DeleteMenuItem: TMenuItem;
    Agent: TAgent;
    AgentPopup: TPopupMenu;
    ReadMenuItem: TMenuItem;
    PauseMenuItem: TMenuItem;
    HideMenuItem: TMenuItem;
    HelpMenuItem: TMenuItem;
    AboutMenuItem: TMenuItem;
    ContentsMenuItem: TMenuItem;
    N1: TMenuItem;
    AddBookMarkMenuItem2: TMenuItem;
    ReadFromCaretMenuItem2: TMenuItem;
    ChangeFontMenuItem2: TMenuItem;
    N2: TMenuItem;
    DeleteBookmarkMenuItem2: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure TreeViewClick(Sender: TObject);
    procedure ChangeFontMenuItemClick(Sender: TObject);
    procedure AddBookMarkMenuItemClick(Sender: TObject);
    procedure ReadFromCaretMenuItemClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AgentRequestComplete(Sender: TObject; Request: IDispatch);
    procedure OpenBookMenuItemClick(Sender: TObject);
    procedure SelectAgentMenuItemClick(Sender: TObject);
    procedure AgentShutdown(Sender: TObject);
    procedure AgentHide(Sender: TObject; const CharacterID: WideString;
      Cause: Smallint);
    procedure ExitMenuClickClick(Sender: TObject);
    procedure SaveBooksettingsMenuItemClick(Sender: TObject);
    procedure ChangeSpeedMenuItemClick(Sender: TObject);
    procedure DeleteMenuItemClick(Sender: TObject);
    procedure AgentClick(Sender: TObject; const CharacterID: WideString;
      Button, Shift, x, y: Smallint);
    procedure ReadMenuItemClick(Sender: TObject);
    procedure PauseMenuItemClick(Sender: TObject);
    procedure HideMenuItemClick(Sender: TObject);
    procedure ContentsMenuItemClick(Sender: TObject);
    procedure AboutMenuItemClick(Sender: TObject);


  private
    DoubleNewLine : String;
    Speed         : integer;
    IlFileIni     : TIniFile;
    procedure ReadNextSentence;
    procedure ReadIni;
    procedure WriteIni;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  Puppet : IAgentCtlCharacterEx;
  SpeakRequest : IAgentCtlRequest;
  CurrentPuppet : String;
  ReturnString : String;
  AgentInstalled : boolean;

implementation

uses SpeedUnit, Unit1, UAboutBox;

{$R *.DFM}


procedure TMainForm.FormCreate(Sender: TObject);
begin
  ReadIni;
  SetLength(ReturnString,1);
  ReturnString[1] := Chr(13);
  GutenbergMemo.TreeView := TreeView;
  if (GutenbergMemo.Filename <> '') and (GutenbergMemo.Open) then
    GutenbergMemo.SelStart := IlFileIni.ReadInteger('Main', GutenbergMemo.FileName, 1);
  Speed := 140;
  TreeView.Repaint;

  try
    if (CurrentPuppet <> '') then begin
      Agent.OnShutDown := AgentShutDown;
      Agent.Connected := True;
      Agent.Characters.Load(CurrentPuppet, CurrentPuppet);
      Puppet := Agent.Characters.Character(CurrentPuppet);
      Puppet.AutoPopupMenu := false;
      AgentInstalled := true;
    end
  except on Exception do
    begin
      AgentInstalled := false;
      ReadFromCaretMenuItem.Enabled := False;
      //SelectAgentMenuItem.Enabled  := False;
      if MessageDlg('Microsoft Agent was not installed. Do you want to get it now?', mtWarning, [mbYes, mbNo], 0) = mrYes then
        ShowWebPage('http://www.msagentring.org');
    end; { begin }
  end; { except }
  SetLength(DoubleNewLine,2);
  DoubleNewLine[1] := Chr(13);
  DoubleNewLine[2] := Chr(13);
end;

procedure TMainForm.TreeViewClick(Sender: TObject);
var LastCharNo : integer;
begin
  GutenbergMemo.GotoLine(-1);
  LastCharNo := TreeView.GetIndex;
  GutenbergMemo.GotoLine(LastCharNo);
end;

procedure TMainForm.ChangeFontMenuItemClick(Sender: TObject);
begin
  if FontDialog.Execute then
    GutenbergMemo.Font := FontDialog.Font;

end;

procedure TMainForm.AddBookMarkMenuItemClick(Sender: TObject);
var BookMark : String;
begin
  with TreeView do begin
    InsertBookMarkName.ShowModal;
    BookMark := InsertBookMarkName.Bookmark;
    if Length(BookMark) > 0 then begin
      Selected := GetChapter(GutenBergMemo.SelStart);
      AddBookMark(Selected, GutenBergMemo.SelStart, BookMark);
    end;
    TreeView.SetFocus;
  end;
end;

procedure TMainForm.ReadFromCaretMenuItemClick(Sender: TObject);
begin
  try
    Puppet.Show(False);
    Puppet.Play('Read');
  except on Exception do
  end;
  ReadNextSentence;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if AgentInstalled and (CurrentPuppet <> '') then Agent.Characters.Unload(CurrentPuppet);
  SaveBookSettingsMenuItemClick(self);
  WriteIni;
end;

procedure TMainForm.ReadNextSentence;

  function CutNewLines(InString : String) : String;
  var i : integer;
  begin
    for i := 1 to Length(InString) do
      if InString[i] = Chr(13) then
        InString[i] := ' ';
    result := InString;
  end;

  function GetFirstSeparator : integer;
  var  FirstPoint, FirstDl : integer;
  begin
    with GutenbergMemo do begin
      FirstPoint := SelStart;
      repeat
        FirstPoint  := FindText('.',FirstPoint+1,Length(Text)-FirstPoint-1,[]);
      until (CharsAt(FirstPoint-2,FirstPoint) <> 'Mr.') and
            (CharsAt(FirstPoint-3, FirstPoint) <> 'Mrs.') and
            (CharsAt(FirstPoint-4, FirstPoint) <> 'Miss.'); // I know, it sucks.
      FirstDl   :=  FindText(DoubleNewLine,SelStart+1,Length(Text)-SelStart-1,[]);
    end;
    if FirstDl < FirstPoint then
      result := FirstDl
    else
      result := FirstPoint;
  end;

var
  FirstSeparator, FirstSpace, FirstReturn: integer;
  Sentence   : String;

begin
  with GutenbergMemo do begin
    FirstSeparator := GetFirstSeparator;
    SelLength  := FirstSeparator - SelStart;
    if (Pos('CHAPTER', SelText) > 0) or (Pos('Chapter', SelText) > 0) then begin
      FirstSpace :=  FindText(' ',SelStart+9,Length(Text)-SelStart-9,[]);
      FirstReturn := FindText(ReturnString,SelStart+9,Length(Text)-SelStart-9,[]);
      if FirstSpace < FirstReturn then
        FirstSeparator := FirstSpace
      else
        FirstSeparator := FirstReturn;
      SelLength := FirstSeparator-SelStart;
      Sentence := 'Chapter ' + RomanToArabic(Copy(SelText,8,SelLength-5))
    end else
      Sentence := LowerCase(SelText);
      Sentence := '\spd='+IntToStr(Speed)+'\'+CutNewLines(Sentence);
      LastSelected := FirstSeparator+1;
      if (Sentence = '') then
        ReadNextSentence
      else
        SpeakRequest := Puppet.Speak(Sentence,'');
  end;
end;

procedure TMainForm.AgentRequestComplete(Sender: TObject; Request: IDispatch);
begin
  if (Request = SpeakRequest) and (Puppet.Visible) then begin
    GutenbergMemo.SelStart := GutenbergMemo.LastSelected;
    ReadNextSentence;
  end;
end;

procedure TMainForm.OpenBookMenuItemClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    GutenbergMemo.FileName := OpenDialog.FileName;
  GutenBergMemo.Open;
  if AgentInstalled and (CurrentPuppet <>  '') then begin
    Agent.Characters.Unload(CurrentPuppet);
    Agent.Characters.Load(CurrentPuppet, CurrentPuppet);
    Puppet := Agent.Characters.Character(CurrentPuppet);
    Puppet.AutoPopupMenu := False;
  end;
  SaveBooksettingsMenuItemClick(self);
end;

procedure TMainForm.SelectAgentMenuItemClick(Sender: TObject);
var PuppetFileName : String;
begin
  if {(SpeakRequest = nil) and} OpenDialogAgent.Execute then  begin
    PuppetFileName := OpenDialogAgent.FileName;
    if PuppetFileName <> CurrentPuppet then begin
      if CurrentPuppet <> '' then
        Agent.Characters.Unload(CurrentPuppet);
      CurrentPuppet := PuppetFileName;
      Agent.Characters.Load(CurrentPuppet, CurrentPuppet);
      Puppet := Agent.Characters.Character(CurrentPuppet);
      Puppet.AutoPopupMenu := false;
      Puppet.Show(False);
      try
        Puppet.Play('Read');
      except on Exception do end;
    end;
  end;
end;

procedure TMainForm.AgentShutdown(Sender: TObject);
begin
  SpeakRequest := nil;
end;

procedure TMainForm.AgentHide(Sender: TObject; const CharacterID: WideString;
  Cause: Smallint);
begin
  SpeakRequest := nil;
end;

procedure TMainForm.ExitMenuClickClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.SaveBooksettingsMenuItemClick(Sender: TObject);
begin
  GutenbergMemo.Save;
  IlFileIni.WriteInteger('Main', GutenbergMemo.FileName, GutenbergMemo.SelStart);
end;

procedure TMainForm.WriteIni;
begin
  IlFileIni.WriteInteger('Main','FontHeight', GutenbergMemo.Font.Height);
  IlFileIni.WriteString('Main','FontName', GutenbergMemo.Font.Name);
  IlFileIni.WriteInteger('Main','LeftPanelWidth', LeftPanel.Width);
  IlFileIni.WriteString('Main','FileName', GutenbergMemo.FileName);
  IlFileIni.WriteInteger('Main','Height', Height);
  IlFileIni.WriteString('Main', 'Puppet', CurrentPuppet);
  IlFileIni.WriteInteger('Main','Left', Left);
  IlFileIni.WriteInteger('Main','Width', Width);
  IlFileIni.WriteInteger('Main','Top', Top);
end;

procedure TMainForm.ReadIni;
begin
  IlFileIni := TIniFile.Create(GetWinDir+'\Gutenberg.ini');
  GutenbergMemo.Font.Height := IlFileIni.ReadInteger('Main', 'FontHeight', 10);
  GutenbergMemo.Font.Name   := IlFileIni.ReadString('Main', 'FontName', 'Courier');
  LeftPanel.Width           := IlFileIni.ReadInteger('Main', 'LeftPanelWidth', 385);
  Height                    := IlFileIni.ReadInteger('Main', 'Height',  550);
  GutenbergMemo.FileName    := IlFileIni.ReadString('Main', 'FileName', '');
  CurrentPuppet             := IlFileIni.ReadString('Main', 'Puppet', '');
  Left                      := IlFileIni.ReadInteger('Main','Left', Left);
  Top                       := IlFileIni.ReadInteger('Main','Top',Top);
  Width                     := IlFileIni.ReadInteger('Main','Width',Width);
end;

procedure TMainForm.ChangeSpeedMenuItemClick(Sender: TObject);
begin
  SpeedDialog.ScrollBar.Position := Speed;
  SpeedDialog.ShowModal;
  Speed :=  SpeedDialog.ScrollBar.Position;
end;

procedure TMainForm.DeleteMenuItemClick(Sender: TObject);
begin
  if (TreeView.Selected <> nil) then
    TreeView.DeleteBookMark(TreeView.Selected);
end;

procedure TMainForm.AgentClick(Sender: TObject; const CharacterID: WideString;
  Button, Shift, x, y: Smallint);
begin
  if Button = 2 then
    AgentPopup.Popup(x, y);
end;

procedure TMainForm.ReadMenuItemClick(Sender: TObject);
begin
  try
      Puppet.Play('Read');
  except on Exception do end;
  ReadNextSentence;
end;

procedure TMainForm.PauseMenuItemClick(Sender: TObject);
begin
  try
    Puppet.Play('ReadReturn');
  except on Exception do end;
  SpeakRequest := nil;
end;

procedure TMainForm.HideMenuItemClick(Sender: TObject);
begin
  Puppet.Play('Hide');
  SpeakRequest := nil;
end;

procedure TMainForm.ContentsMenuItemClick(Sender: TObject);
begin
  ShowWebPage('Help.htm');
end;

procedure TMainForm.AboutMenuItemClick(Sender: TObject);
begin
  AboutBox.Show;
end;

end.
