unit utenbergMemo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, RxRichEd, comctrls, GutenbergTreeView;

type
  TGutenbergMemo = class(TRxRichEdit)
    constructor Create(AOwner : TComponent); override;
    procedure Replace(Source, Destination : String);
    function CharsAt(first, last : integer) : String;
  private
    FFileName : String;
  protected
    { Protected declarations }
  public
    TreeView : TGutenbergTreeView;
    LastSelected : Integer;
    function Open : boolean;
    procedure GoToLine(CharNo :integer);
    procedure Save;

  published
    property FileName : String read FFileName write FFileName;
  end;

procedure Register;

implementation

constructor TGutenbergMemo.Create(AOwner : TComponent);
begin
  inherited;
end;
procedure TGutenbergMemo.GoToLine(CharNo : integer);
begin
  {set the cursor (caret) to that position}
  Perform(EM_SetSel, CharNo, CharNo);
  {scroll the cursor into view}
  Perform(EM_ScrollCaret, 0, 0);
  SetFocus;
  {set the focus so that the cursor shows}
end;

function TGutenbergMemo.CharsAt(first, last : integer) : String;
var OldSelStart : integer;
begin
  OldSelStart := SelStart;
  SelStart := first;
  SelLength := last-first+1;
  result := SelText;
  SelStart := OldSelStart;
end;

procedure TGutenbergMemo.Replace(Source, Destination : String);
var OldSelStart : integer;
begin
  OldSelStart:= SelStart;
  SelStart  := FindText(source,SelStart,Length(Text)-SelStart,[]);
  SelLength := Length(Source);
  SelText := Destination;
  SelStart := OldSelStart;
end;


function TGutenbergMemo.Open : boolean;
var Line : String;
  EndIndex : Integer;
  ConfigFile : String;

begin
  try
    Lines.LoadFromFile(FileName);
    EndIndex := FindText('*END*',1,Length(Text)-1,[]);
    SelStart  := 1;
    SelLength := EndIndex;
    Line := GetSelText;
    ClearSelection;
    Lines.Delete(0);
    Lines.Delete(0); ///hmmmm.....
    TreeView.Items.Clear;
    ConfigFile := ChangeFileExt(FFileName,'.bmk');
    if FileExists(ConfigFile) then
      TreeView.LoadFromFile(ConfigFile)
    else
      TreeView.AddNodes(self);
    result := true;
  except on Exception do
    result := false;
  end;
end;

procedure TGutenbergMemo.Save;
var ConfigFile : String;
begin
  ConfigFile := ChangeFileExt(FFileName,'.bmk');
  TreeView.SaveToFile(ConfigFile);
  TreeView.Repaint;
end;

procedure Register;
begin
  RegisterComponents('Gutenberg', [TGutenbergMemo]);
end;

end.
